/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++                 LS_add-on.cpp                  +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++         Copyright © 2007 Ove Peter.            +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

#include <add-ons/tracker/TrackerAddOn.h>
#include <app/Roster.h>
#include <interface/Alert.h>

#include "LS_add-on.h"

/*------------------------ Tracker add-on ------------------------------*/

void process_refs(entry_ref dir_ref, BMessage *message, void *)
{
	entry_ref	ref;
	status_t	roster_stat = B_ERROR;
	
	/* ---> if true we got fileref(s), send msg directly to app --- */
	if(message->FindRef("refs", &ref) == B_OK)
	{	/* -- BUT: if the ref is the only one and it is a DIR, send special workdir msg -- */
		if(message->FindRef("refs", 1, &ref) != B_OK)
		{
			if(message->FindRef("refs", 0, &ref) == B_OK)	// for safety, to really have the 1st ref.
			{
				BEntry	anEntry(&ref, true);
				if(anEntry.IsDirectory())
				{
					BMessage	*msg = new BMessage(B_REFS_RECEIVED);
					msg->AddRef("dirref", &ref);
					roster_stat = be_roster->Launch(APP_SIG, msg, NULL);
					delete msg;
				} else {
					roster_stat = be_roster->Launch(APP_SIG, message, NULL);
				}
			}
		} else {
			roster_stat = be_roster->Launch(APP_SIG, message, NULL);
		}
	} else {
	/* -- Examine dir_ref (msg from Tracker when invoked in a window with no files selected -- */
		BEntry	anEntry(&dir_ref, true);
		if(anEntry.IsDirectory())
		{	/* ---> if it contained a dir, send special "dirref" msg for changing workdir --- */
			BMessage	*msg = new BMessage(B_REFS_RECEIVED);
			msg->AddRef("dirref", &dir_ref);
			roster_stat = be_roster->Launch(APP_SIG, msg, NULL);
			delete msg;
		}
	}

	/* ---> Display error condition, if any --- */
	if( (roster_stat != B_OK) && (roster_stat != B_ALREADY_RUNNING) )
	{
		const char	*alerttext = "Little Shell Tracker add-on:\n\n"
					"Can not launch the main Little Shell Application! Please check if it is installed.\n";
		BAlert		*alert = new BAlert("Add-On Error", alerttext, "Damn!", NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT);
		alert->Go();
	}
}


/*------------------------- BApplication -------------------------------*/

int main( void )
{
	LittleShellAddOn *theApp;

    theApp = new LittleShellAddOn();
    theApp->Run();
	
    delete(theApp);
    return(0);
}


LittleShellAddOn::LittleShellAddOn( void ):BApplication(ADD_ON_SIG)
{	
	int32		areply;
	const char	*alerttext = "Tracker add-on for LittleShell.\n\n"
					"To install, please move it to\n\"/boot/home/config/add-ons/Tracker\"\n\n"
					"This add-on can't be used without the installed main LittleShell application.\n";

	BAlert		*alert = new BAlert("Little Shell add-on", alerttext, "Help", "OK", NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT);

	alert->SetShortcut(1, B_ESCAPE);
	areply = alert->Go();
	
	if(areply == 0) {	// Show help
		BMessage	msg(B_ARGV_RECEIVED);
		msg.AddString("argv", "app");
		msg.AddString("argv", T_ADDON_HELP);
		msg.AddInt32("argc", 2);
		be_roster->Launch("text/html", &msg, NULL);
	}
	
  	be_app->PostMessage(B_QUIT_REQUESTED);
}


LittleShellAddOn::~LittleShellAddOn( void )
{

}
