/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++                PrefWindow.cpp                  +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++         Copyright © 2007 Ove Peter.            +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

#include <app/Application.h>
#include <interface/Box.h>
#include <interface/Button.h>
#include <interface/TextControl.h>
#include <interface/PopUpMenu.h>
#include <interface/MenuItem.h>
#include <interface/MenuField.h>
#include <interface/StringView.h>
#include <stdio.h>			// for sprintf()
#include <stdlib.h>			// for atol()

#include "PrefWindow.h"
#include "ShellWindow.h"
#include "../olib/Colors.h"


PrefWindow::PrefWindow(BRect frame, Preferences *preferences)
	: BWindow(frame, "LittleShell Preferences", B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE | B_ASYNCHRONOUS_CONTROLS | B_WILL_ACCEPT_FIRST_CLICK)
{
	int32	i = 0; // for various loops
	
	/*---------------------- Load preferences -----------------------------*/
		
	// Reload the preferences
	prefs = preferences;
	prefs->Load();
	
	/* --- Chosen Tab for this window! --- */
	int32	tabSelect = 0;
	if(prefs->FindInt32("prefwin_tabselect", &tabSelect) != B_OK)
		tabSelect = 0;

	// Wether to append an ampersand to shell commands
	if(prefs->FindBool("addamper", &addAmper) != B_OK)
		addAmper = false;

	// Wether to open a window for stdout of shell commands
	if(prefs->FindBool("showstdout", &showStdout) != B_OK)
		showStdout = true;
		
	// Wether to show the command as stdout window title
	if(prefs->FindBool("stdouttitle", &stdoutTitle) != B_OK)
		stdoutTitle = true;

	// Wether ShellWindow should Stay On Top
	if(prefs->FindBool("stayontop", &stayOnTop) != B_OK)
		stayOnTop = false;
	
	// Quit after execution or stay open 
	if(prefs->FindBool("autoclose", &autoClose) != B_OK)
		autoClose = false;

	// Restrict history to maxHistItems entries
	if(prefs->FindInt32("maxhistory", &maxHistItems) != B_OK)
		maxHistItems = MAX_HIST_ITEMS;

	// Wether to show the Work Directory in Title
	if(prefs->FindBool("wdirtitle", &showWorkDir) != B_OK)
		showWorkDir = true;
	
	// Wether to save the last WorkDir
	if(prefs->FindBool("saveworkdir", &saveWorkDir) != B_OK)
		saveWorkDir = true;
		
	// Get Main Window look
	if(prefs->FindString("windowlook", &windowLook) != B_OK)
		windowLook = "Standard";


	/*------------------- Grey background top view and Font------------------------*/
	
	BView *back = new BView(Bounds(), "PrefWinBack", B_FOLLOW_ALL, B_WILL_DRAW);
	back->SetViewColor(greyCol);
	back->SetLowColor(greyCol);
	
	BFont	font(be_plain_font);
	if(font.Size() > 12.0)			
		font.SetSize(12.0);
	back->SetFont(&font, B_FONT_ALL);
	
	/*------------------------- Tab Pref Panels ---------------------------*/
	
	BRect r = Bounds();
	r.InsetBy(5, 5);
	r.bottom -= 35;		// Make space for the buttons
	
	tabView = new BTabView(r, "tab_view");
	tabView->SetViewColor(greyCol);
	
	r = tabView->Bounds();
	r.InsetBy(5, 5);
	r.bottom -= tabView->TabHeight();
	
	float width = r.right;
	float height = r.bottom;
	
	genView = new BView(r, NULL, B_FOLLOW_NONE, B_WILL_DRAW | B_NAVIGABLE);
	genView->SetViewColor(greyCol);
	genView->SetFont(&font, B_FONT_ALL);
	genTab = new BTab();
	tabView->AddTab(genView, genTab);
	genTab->SetLabel("Commands");
	
	historyView = new BView(r, NULL, B_FOLLOW_NONE, B_WILL_DRAW | B_NAVIGABLE);
	historyView->SetViewColor(greyCol);
	historyTab = new BTab();
	tabView->AddTab(historyView, historyTab);
	historyTab->SetLabel("General");
			
	appearanceView = new BView(r, NULL, B_FOLLOW_NONE, B_WILL_DRAW | B_NAVIGABLE);
	appearanceView->SetViewColor(greyCol);
	appearanceTab = new BTab();
	tabView->AddTab(appearanceView, appearanceTab);
	appearanceTab->SetLabel("Appearance");
	

	/* ---------------------- Shell Command Tab ---------------------- */

	// Shell commands BBox	
	BRect	scRect(0, 5.0, width-5.0, height-5.0);
	
	BBox	*scBox = new BBox(scRect,
					"shellcommbox",
					B_FOLLOW_ALL,
					B_WILL_DRAW | B_NAVIGABLE,
					B_FANCY_BORDER);
	scBox->SetLabel("Shell commands");
	genView->AddChild(scBox);

	// Append an amper to shell commands CheckBox
	BRect shellRect(5.0, 15.0, width-10.0, 35.0);
	amperCB = new OCheckBox(shellRect, "amperCB", "Automatically append \"&\" to shell commands.", new BMessage(AMPERCB), B_FOLLOW_NONE, B_NAVIGABLE | B_WILL_DRAW);
	amperCB->SetValue(addAmper);
	scBox->AddChild(amperCB);

	// Show stdout CheckBox
	shellRect.top += 20.0;	// TODO: + 25 would be nicer, but the window has to be bigger then
	shellRect.bottom += 20.0;
	stdoutCB = new OCheckBox(shellRect, "stdout", "Show stdout and stderr (if any output) in window", new BMessage(SHOWOUTCB), B_FOLLOW_NONE, B_NAVIGABLE | B_WILL_DRAW);
	stdoutCB->SetValue(showStdout);
	scBox->AddChild(stdoutCB);

	// Show stdout Title CB
	shellRect.top += 20.0;
	shellRect.bottom += 20.0;
	shellRect.left += 5.0;
	outTitleCB = new OCheckBox(shellRect, "outtitle", "Stdout window title is the command", new BMessage(OUTTITLECB), B_FOLLOW_NONE, B_NAVIGABLE | B_WILL_DRAW);
	outTitleCB->SetValue(stdoutTitle);
	outTitleCB->SetEnabled(showStdout);	// enabled if ShowStdout is also true
	scBox->AddChild(outTitleCB);
	
	// Status Text View at the bottom
	BRect	statRect(5.0, height-30.0, width-10.0, height-15.0);
	shellStat = new BStringView(statRect, "amperStatView", NULL, B_FOLLOW_NONE, B_WILL_DRAW);
	shellStat->SetAlignment(B_ALIGN_LEFT);
	// shellStat->SetHighColor(redCol);
	// shellStat->SetText("This window badly needs a redesign :)");
	scBox->AddChild(shellStat);
	
		
	/*-------------------------- History Tab -----------------------------*/

	// Maximum History items BBox	
	BRect	maxHistRect(0, 5.0, width-5.0, height-65.0);	
	BBox	*maxHistBox = new BBox(maxHistRect, "maxhistbox", B_FOLLOW_ALL, B_WILL_DRAW | B_NAVIGABLE, B_FANCY_BORDER);
	maxHistBox->SetLabel("Command History list");
	historyView->AddChild(maxHistBox);
	

	// Input Text Control
	// Convert the int32 maxHistItems Value to a a string first, so that it can be displayed in TextControl
	char	tempMaxHist[128];
	sprintf(tempMaxHist, "%li", maxHistItems);

	histTextBar = new BTextControl(BRect(5.0, 20.0, 220.0, 30.0),
							 	"max_hist_text", 
								"Enter maximum history items:",
								tempMaxHist,
								new BMessage(HIST_TEXT_MSG),
								B_FOLLOW_ALL,
								B_WILL_DRAW | B_NAVIGABLE);
	histTextBar->SetDivider(170.0);
	maxHistBox->AddChild(histTextBar);


	// Maximum History Menu
	maxHistMenu = new BPopUpMenu(tempMaxHist);
	
	for(i = 0; i<=150; i+=10) {
		BMessage *maxMsg = new BMessage(MENU_MAX_HIST);
		maxMsg->AddInt32("max_hist_int32", i);
		sprintf(tempMaxHist, "%li", i);
		maxHistMenu->AddItem(new BMenuItem(tempMaxHist, maxMsg));
	}
	
	maxMField = new BMenuField(BRect(width-70.0, 18, width-7.0, 25), NULL, NULL, maxHistMenu);
	maxMField->SetDivider(0);
	maxHistBox->AddChild(maxMField);


	// Another box like a line
	BRect	lineRect(10.0, 53.0, width-15.0, 54.0);
	BBox	*lineBox = new BBox(lineRect, "linebox", B_FOLLOW_ALL, B_WILL_DRAW | B_NAVIGABLE, B_FANCY_BORDER);
	maxHistBox->AddChild(lineBox);	

	// Clear History Now -Button
	deleteHistB = new BButton(BRect(width/2, 62.0, width-13.0, 72.0), NULL, "Clear history NOW", new BMessage(CLEAR_HIST_NOW));
	maxHistBox->AddChild(deleteHistB);
	
	// Clear History String Status View
	clearHistView = new BStringView(BRect(5.0, 67.0, (width/2)-10.0, 85.0), "clearHistView", NULL, B_FOLLOW_NONE, B_WILL_DRAW);
	clearHistView->SetAlignment(B_ALIGN_CENTER);
	clearHistView->SetHighColor(redCol);	// red text
	BString tmpString;
	int32	currItems = 0;
	for(i=0; prefs->FindString("history", i, &tmpString) == B_OK; i++) {
		currItems++;
	}	// Display current No. of items in history list
	tmpString = "Current items: ";
	tmpString << (int32 )currItems;
	clearHistView->SetText(tmpString.String());
	maxHistBox->AddChild(clearHistView);


	/* --- General Box --- */
	BRect	genRect(0, height-60.0, width-5.0, height-5.0);
	BBox	*genBox = new BBox(genRect,
					"genbox",
					B_FOLLOW_ALL,
					B_WILL_DRAW | B_NAVIGABLE,
					B_FANCY_BORDER);
	genBox->SetLabel("General");
	historyView->AddChild(genBox);
	
	BRect genCBR(5.0, 15.0, width-10.0, 35.0);
	autoCloseCB = new OCheckBox(genCBR, "autocloseCB", "Automatically close window after process is done", new BMessage(AUTOCLOSECB), B_FOLLOW_NONE, B_NAVIGABLE | B_WILL_DRAW);
	autoCloseCB->SetValue(autoClose);
	genBox->AddChild(autoCloseCB);
	
	genCBR.top += 18;
	genCBR.bottom += 18;
	saveWorkDirCB = new OCheckBox(genCBR, "saveworkdirCB", "Remember last used work directory", new BMessage(SAVEDIRCB), B_FOLLOW_NONE, B_NAVIGABLE | B_WILL_DRAW);
	saveWorkDirCB->SetValue(saveWorkDir);
	genBox->AddChild(saveWorkDirCB);


	/*------------------------ Appearance Tab ----------------------------*/

	// Shell commands BBox	
	BRect	guiBoxRect(0, 2.0, width-5.0, height-5.0);
	
	BBox	*guiBox = new BBox(guiBoxRect, "guibox", B_FOLLOW_ALL, B_WILL_DRAW | B_NAVIGABLE, B_FANCY_BORDER);
	guiBox->SetLabel("Interface look & behaviour");
	appearanceView->AddChild(guiBox);
	
	// Window look Menu
	winLookMenu = new BPopUpMenu(windowLook.String());

	const char *theLooks[5] = {"Standard", "Titled", "Mini-Tab", "Tabless", "Borderless"};
	for(i = 0; i<5; i++)
	{
		BMessage	*lookMsg = new BMessage(WIN_LOOK_SELECTED);
		lookMsg->AddString("windowlook", theLooks[i]);
		winLookMenu->AddItem(new BMenuItem(theLooks[i], lookMsg));
	}

	BRect	guiRect(5.0, 20.0, width-7.0, 30.0);
	winLookMF = new BMenuField(guiRect,
							"WindowLook", 
							"Main window look:", 
							winLookMenu,
							false,
							B_FOLLOW_NONE, 
							B_WILL_DRAW | B_NAVIGABLE);
	winLookMF->SetDivider(130.0);
	guiBox->AddChild(winLookMF);

	guiRect.bottom += 30;
	guiRect.top += 30;
	stayTopCB = new OCheckBox(guiRect, "stayTopCB", "Stay On Top (show main windows in all workspaces)", new BMessage(STAYTOPCB), B_FOLLOW_NONE, B_NAVIGABLE | B_WILL_DRAW);
	stayTopCB->SetValue(stayOnTop);
	guiBox->AddChild(stayTopCB);

	guiRect.bottom += 20;
	guiRect.top += 20;
	workDirTitCB = new OCheckBox(guiRect, "workdircb", "Show current work directory in window title", new BMessage(WORKDIRTCB), B_FOLLOW_NONE, B_NAVIGABLE | B_WILL_DRAW);
	workDirTitCB->SetValue(showWorkDir);
	guiBox->AddChild(workDirTitCB);
	
	/* ------------------- Cancel and Save - Buttons -------------------- */

	BRect cancelBox(width-210, height+40, width-110, height+60);
	BRect okayBox(width-100, height+40, width+7, height+60);
	
	cancelB = new BButton(cancelBox, NULL, "Use", new BMessage(PREF_CANCEL));

	okayB = new BButton(okayBox, NULL, "Save", new BMessage(PREF_OKAY));
	okayB->MakeDefault(true);

	back->AddChild(okayB);
	back->AddChild(cancelB);
	
	/* ------------------ Final adjustments, add tab now ------------------ */
	
	back->AddChild(tabView);
	AddChild(back);
	
	tabView->Select(tabSelect);
	Show();
}

PrefWindow::~PrefWindow( void )
{

}

/*-------------------------------------------------------------------------*/

void PrefWindow::MessageReceived(BMessage *message)
{	// TODO: handle the prefs-change-notice to ShellWindows differently, delete new messages
	switch(message->what)
	{
		case PREF_CANCEL:
			prefs->SetInt32("prefwin_tabselect", tabView->Selection());
			prefs->Save();
			Quit();
			break;

		case PREF_OKAY:
			SavePreferences();
			break;

		case AMPERCB:
		{
			if(addAmper == false) addAmper = true;
			else addAmper = false;
			// Update the Status string view and the ShowStdout thingie:
			if(addAmper == true)
				shellStat->SetText("Exit status of programs can't be displayed now!");
			else
				shellStat->SetText("");
			// Update all shell windows
			BMessage	*addAmperMsg = new BMessage(UPDATE_ADD_AMPER);
			addAmperMsg->AddBool("upamper", addAmper);
			be_app->PostMessage(addAmperMsg);
		}
			break;
		
		case SHOWOUTCB:
		{
			if(showStdout == false) showStdout = true;
			else showStdout = false;
			outTitleCB->SetEnabled(showStdout);	// en-/disable the dependant stdout title CB
			BMessage	*stdoutMsg = new BMessage(UPDATE_STDOUT);
			stdoutMsg->AddBool("stdout", showStdout);
			be_app->PostMessage(stdoutMsg);
		}
			break;
		
		case OUTTITLECB:
		{
			if(stdoutTitle == false) stdoutTitle = true;
			else stdoutTitle = false;
			BMessage	*outTitleMsg = new BMessage(UPDATE_OUTTITLE);
			outTitleMsg->AddBool("stdouttitle", stdoutTitle);
			be_app->PostMessage(outTitleMsg);
		}
			break;
		
		case AUTOCLOSECB:
		{
			if(autoClose == false) autoClose = true;
			else autoClose = false;
			BMessage	*autocloseMsg = new BMessage(UPDATE_AUTOCLOSE);
			autocloseMsg->AddBool("autoclose", autoClose);
			be_app->PostMessage(autocloseMsg);
		}			
			break;

		case MENU_MAX_HIST:
		{	/*--- Update the Max Text Control when Max Menu was changed ---*/
			message->FindInt32("max_hist_int32", &maxHistItems);
			// convert int32 to char	
			char	buff[128];
			sprintf(buff, "%li", maxHistItems);
			// change the text control text
			histTextBar->SetText(buff);
		}
			break;
		
		case HIST_TEXT_MSG:
		{	/*--- Update the Max Menu when Max Text Control was changed ---*/
			const char	*newMaxChar;
			newMaxChar = histTextBar->Text();
			maxHistItems = atol(newMaxChar);
			
			BMessage *maxMsg = new BMessage(MENU_MAX_HIST);
			maxMsg->AddInt32("max_hist_int32", maxHistItems);
			
			BMenuItem	*newMaxItem = new BMenuItem(newMaxChar, maxMsg);
			maxHistMenu->AddItem(newMaxItem);
			newMaxItem->SetMarked(true);
		}
			break;

		case CLEAR_HIST_NOW:
		{	// Clear by setting maxHistItems = 0 first...
			BMessage	*clearNowMsg = new BMessage(UPDATE_MAX_HIST);
			clearNowMsg->AddInt32("maxhistory", 0);
			be_app->PostMessage(clearNowMsg);
			
			snooze(1000);	// just for safety
		
			// ... then revert maxHistItems=0 back to user chosen value
			BMessage	*currMaxValue = new BMessage(UPDATE_MAX_HIST);
			currMaxValue->AddInt32("maxhistory", maxHistItems);
			be_app->PostMessage(currMaxValue);
			
			clearHistView->SetText("Cleared!");
		}
			break;

		case WIN_LOOK_SELECTED:
		{
			message->FindString("windowlook", &windowLook);
			
			BMessage	*winTypeMsg = new BMessage(UPDATE_WIN_LOOK);
			winTypeMsg->AddString("windowlook", windowLook);
			be_app->PostMessage(winTypeMsg);
		}	
			break;
		
		case STAYTOPCB:
		{
			if(stayOnTop == false) stayOnTop = true;
			else stayOnTop = false;			
			
			BMessage	*stayTopMsg = new BMessage(UPDATE_STAY_TOP);
			stayTopMsg->AddBool("stayontop", stayOnTop);
			be_app->PostMessage(stayTopMsg);
		}
			break;
			
		case WORKDIRTCB:
		{
			if(showWorkDir == false) showWorkDir = true;
			else showWorkDir = false;
			
			BMessage	*showWDirMsg = new BMessage(UPDATE_SHOW_WDIR);
			showWDirMsg->AddBool("wdirtitle", showWorkDir);
			be_app->PostMessage(showWDirMsg);
		}
			break;
		
		case SAVEDIRCB:
		{
			if(saveWorkDir == false) saveWorkDir = true;
			else saveWorkDir = false;
			
			BMessage	*saveWDirMsg = new BMessage(UPDATE_SAVE_WDIR);
			saveWDirMsg->AddBool("saveworkdir", saveWorkDir);
			be_app->PostMessage(saveWDirMsg);
		}
			break;
			
		default:
			BWindow::MessageReceived(message);
			break;
	}
}


void PrefWindow::SavePreferences( void )
{
	// First get and save maximum history list bool
	const char	*maxHistChar;
	maxHistChar = histTextBar->Text();
	maxHistItems = atol(maxHistChar);

	prefs->SetInt32("maxhistory", maxHistItems);

	// Update all window's History menus NOW
	BMessage	*updateMaxMsg = new BMessage(UPDATE_MAX_HIST);
	updateMaxMsg->AddInt32("maxhistory", maxHistItems);
	be_app->PostMessage(updateMaxMsg);
	snooze(2000);	// just for safety
			
	// add amper bool
	prefs->SetBool("addamper", addAmper);
	// Show stdout
	prefs->SetBool("showstdout", showStdout);
	// Stdout title
	prefs->SetBool("stdouttitle", stdoutTitle);
	// StayOnTop
	prefs->SetBool("stayontop", stayOnTop);
	// Auto Close
	prefs->SetBool("autoclose", autoClose);
	// window look
	prefs->SetString("windowlook", windowLook.String());
	// Show Work Dir as Title
	prefs->SetBool("wdirtitle", showWorkDir);
	// Remember last used Workdir
	prefs->SetBool("saveworkdir", saveWorkDir);
	
	// Current chosen Tab
	prefs->SetInt32("prefwin_tabselect", tabView->Selection());
	
	prefs->Save();
	Quit();
}
