/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++                ShellWindow.h                   +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++         Copyright © 2007 Ove Peter.            +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

#ifndef SHELL_WINDOW_H
#define SHELL_WINDOW_H

#include <interface/Window.h>
#include <support/String.h>

#include "Preferences.h"
#include "TopView.h"
#include "StatusView.h"
#include "TextWindow.h"
#include "../olib/OMenuBar.h"
#include "../olib/OTextControl.h"

#define MAX_HIST_ITEMS		50
#define DEF_WIN_TYPE		B_DOCUMENT_WINDOW

#define BUBBLE				'bubh'
#define	HIST_ITEM_SELECTED	'hisi'
#define ZOOM_NOW			'zoom'
#define MINI_NOW			'mini'
#define AUTO_CLOSE			'aucl'
#define STAY_ON_TOP			'stot'
#define	SHOW_STDOUT			'sout'
#define OPEN_FILES			'opfp'
#define	NEW_SHELL_WINDOW	'nwin'
#define QUIT_APP			'quia'
#define OPEN_PREF_WIN		'opwi'
#define UPDATE_HIST_MENU	'uphi'
#define UPDATE_MAX_HIST		'umax'
#define UPDATE_WIN_LOOK		'upwl'
#define UPDATE_ADD_AMPER	'upaa'
#define UPDATE_STDOUT		'upso'
#define UPDATE_STAY_TOP		'upst'
#define UPDATE_AUTOCLOSE	'upac'
#define UPDATE_OUTTITLE		'uoti'
#define UPDATE_SHOW_WDIR	'uwdt'
#define	UPDATE_SAVE_WDIR	'usad'
#define FIND_PANEL			'Tfnd'	// special Message for Tracker (open FindPanel)


class ShellWindow : public BWindow {

	public:
		/* --- Constructor, Hook functions, etc. --- */
						ShellWindow(BRect frame, uint32 windowNumber, BString homedir);
						~ShellWindow( void );

		virtual	bool	QuitRequested( void );							// All prefs except history and workDir are saved here
		virtual void	MessageReceived(BMessage *message);
		/* --- Own functions --- */
		status_t		WorkDirCheckSet(BString dirToCheck, bool alsoSet = true);	// Checks a dir for existance, and by default also SETS the global datamember if valid
		virtual void	AddFileRefs(BMessage *message);					// Adds received files or dirs, does all checks, sets workDir
		virtual void	DisplayStatus(const char *statText, const rgb_color statCol, bool append = false);	// Public function to update the status view
		virtual void	Execute(BMessage *message);						// The main "Execute command" routine
		status_t		SearchCD(BString commandString);				// Checks for and sets (via WorkDirCheckSet()) Change Directory commands
		status_t		SearchFile(BString fileString);					// Checks if command is a filename, and launches the file if yes
		status_t		SearchURLs(BString urlString);					// Checks for URL's in command string
		status_t		LaunchURLs(BString urlString, BString type);	// Execute URL's
		status_t		SearchTrackerFolder(BString pathString);		// Checks for an OpenTracker folder command and checks/opens the path
		status_t		OpenTrackerFolder(BString pathString);			// Opens a given folder in a Tracker window
		int32			ExecuteShellCommand(BString commandString);		// Execute Shell commands

		/* --- These functions update the window after a preferences change, and are called elsewhere (i.e. constructor) also --- */
		virtual void	SetTheWinType(const char *winTypeChar);		// Sets window type
		virtual void	SetBubbleHelp(bool enable);					// Enables or disables BubbleHelp
		virtual void	UpdateHistory(const char *commandString);	// Updates history menu with new command and does ALL needed checks/sets, also SAVES prefs!
		virtual void	UpdateMaxHist(int32 maxOptions);			// REBUILDS history menu when maximum entries has been changed, SAVES prefs
		virtual void	UpdateAddAmper(bool appendAmper);			// Just sets addAmper datamember
		virtual void	UpdateShowOut(bool showOut);				// Sets showStdout datamember and Menu item
		virtual void	UpdateOutTitle(bool titleOut);				// Just sets showOutTitle datamember
		virtual void	UpdateStayOnTop(bool stayTop);				// Sets stayOnTop, updates Menu item and window feel mode
		virtual void	UpdateAutoClose(bool winaClose);			// Sets autoClose datamber and Menu item
		virtual void	UpdateShowWDir(bool showIt);				// Sets showWorkDir, also updates the window title according to the new bool value
		virtual void	UpdateSaveWDir(bool saveIt);				// Just sets the saveWorkDir datamember

	private:
		/* --- Private functions --- */
		static int32	StatusThread(BRect *statRect);
		static int32	ShellThread(const char *command);					// Acts as thread_func for executing commands in a new system() thread
		virtual void	OpenPrefWin( void );								// Opens the Preferences window
		
		/* --- Private datamembers ---*/
		TopView			*fAView;		// Top View.

		OTextControl 	*commandBar;	// Command Input TextControl, very special own implementation.
		BButton			*okButt;		// OK-Button.
		StatusView		*statusView;	// Status View.
		TextWin			*textWin;		// stdout window.
		Preferences		prefs;			// Preferences class.

		OMenuBar		*menuBar;		// The menu bar		
		BMenuItem		*openFileMI;	// Open File Panel MenuItem
		BMenuItem		*autoCloseMI;	// auto close  MI
		BMenuItem		*stayOnTopMI;	// Stay On Top MI
		BMenuItem		*showOutMI;		// Show Stdout MI
		BMenuItem		*bubbleMI;		// Enable Bubble Help MI
		BMenu			*historyMenu;	// Command History menu
		BMenuField		*histMenuField;	// MenuField of above, sticks in TopView

		BString			lastCommand;	// This string will always hold the last inputted command.
		bool			autoClose;		// Wether to stay open or quit after a command.
		bool			stayOnTop;		// Enables Floating win mode for the Window.
		bool			addAmper;		// Always add an amper "&".
		bool			showStdout;		// Wether the stdout should be shown.
		bool			showOutTitle;	// Wether the stdout window title should reflect the command.
		bool			showBubble;		// Wether Bubble help should be displayed.
		int32			maxHistItems;	// Current allowed maximum history entries.
		BString			winType;		// This will be filled by the PrefWindow with a value of "Standard", etc, and used for setting Window type.
		BString			homeDir;		// User Home dir!
		BString			workDir;		// current Work directory!
		bool			showWorkDir;	// Wether the above should be shown as window title.
		bool			saveWorkDir;	// Wether the last WorkDir should be remembered on start.
		uint32			winNum;			// Number assigned to this window by be_app.
		bool			hist_busy;		// Wether the history is in process of being updated.
};

#endif
