/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++                StatusView.cpp                  +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++         Copyright © 2007 Ove Peter.            +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

#include <interface/Window.h>		// for Window()
#include <support/String.h>

#include "StatusView.h"
#include "../olib/Colors.h"

StatusStringView::StatusStringView(BRect frame, char *name)
		: BStringView(frame, name, NULL, B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE)
{
	fMouseDown = false;	// for safety, so that window will not stick to cursor when no button is pressed
	
	SetAlignment(B_ALIGN_LEFT);
	
	/* --- Set the font type --- */
	BFont	font(be_plain_font);
	if(font.Size() > 11.0)			
		font.SetSize(11.0);
	SetFont(&font, B_FONT_ALL);
}


StatusStringView::~StatusStringView( void )
{

}

/* -------------------------------------------------------------------------- */

void StatusStringView::AllAttached(void)
{	/* --- Update colors after all have been attached --- */
	SetViewColor(greyDawn);
	SetLowColor(greyDawn);
	SetHighColor(greyDawn);
	
	BStringView::AllAttached();
}


void StatusStringView::Draw(BRect updateRect)
{	// Style the view with a round rectangle
	SetHighColor(greyColShade);
	StrokeRoundRect(Bounds(), 2.0, 2.0);
	SetHighColor(blackCol);
	
	BStringView::Draw(updateRect);
}


void StatusStringView::MouseDown(BPoint where)
{
	fMouseDown = true;
	fMouseOffset = where;

	SetMouseEventMask(B_POINTER_EVENTS, B_NO_POINTER_HISTORY|B_SUSPEND_VIEW_FOCUS|B_LOCK_WINDOW_FOCUS );

	BStringView::MouseDown(where);
}


void StatusStringView::MouseUp(BPoint where)
{
	fMouseDown = false;
	BStringView::MouseUp(where);
}


void StatusStringView::MouseMoved( BPoint where, uint32 code, const BMessage *a_message)
{
	if(fMouseDown) {
		BPoint win_pos = Window()->Frame().LeftTop();
		Window()->MoveTo( win_pos.x + where.x - fMouseOffset.x, win_pos.y + where.y - fMouseOffset.y );
	} else {
		BStringView::MouseMoved(where, code, a_message);
	}
}

/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

StatusView::StatusView(BRect frame, char *name, rgb_color back_col)
	: BView(frame, name, B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW)
{
	fMouseDown = false;
	
	SetViewColor(back_col);
	
	/* --- Add the text status bar ---*/
	BRect		statTextRect = Bounds();
	statTextRect.left = 28.0;
	
	statusTView = new StatusStringView(statTextRect, "statusTextView");
	AddChild(statusTView);
}

StatusView::~StatusView( void )
{

}

/*-------------------------------------------------------------------------*/

void StatusView::Draw(BRect updateRect)
{	// Style the view with 3 rectangles (led-look)
	BRect	aRect(0, 0, 27, Bounds().Height());
	SetHighColor(veryDarkGrey);
	StrokeRoundRect(aRect, 2.0, 2.0);
	SetHighColor(blackCol);

	for(int32 i = 0; i < 2; i++) {
		aRect.InsetBy(2.0, 2.0);
		StrokeRoundRect(aRect, 2.0, 2.0);
	}	
		
	BView::Draw(updateRect);
}


void StatusView::MouseDown(BPoint where)
{
	fMouseDown = true;
	fMouseOffset = where;

	SetMouseEventMask(B_POINTER_EVENTS, B_NO_POINTER_HISTORY|B_SUSPEND_VIEW_FOCUS|B_LOCK_WINDOW_FOCUS );

	BView::MouseDown(where);
}


void StatusView::MouseUp(BPoint where)
{
	fMouseDown = false;
	BView::MouseUp(where);
}


void StatusView::MouseMoved( BPoint where, uint32 code, const BMessage *a_message)
{
	if(fMouseDown) {
		BPoint win_pos = Window()->Frame().LeftTop();
		Window()->MoveTo( win_pos.x + where.x - fMouseOffset.x, win_pos.y + where.y - fMouseOffset.y );
	} else {
		BView::MouseMoved(where, code, a_message);
	}
}

/* ========================================================================== */

rgb_color StatusView::CurrentColor(void)
{
	return ViewColor();
}


BString StatusView::CurrentText(void)
{
	const char *ctext = statusTView->Text();
	BString currText;
	currText << ctext;
	return currText;
}


void StatusView::NewCol(rgb_color backCol)
{
	SetViewColor(backCol);
	Invalidate();
}


void StatusView::NewText(const char *text)
{
	statusTView->SetText(text);
}


void StatusView::AppendText(const char *text)
{
	BString newText = CurrentText();
	newText << text;
	statusTView->SetText(newText.String());
}
