/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++                 TopView.cpp                    +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++         Copyright © 2007 Ove Peter.            +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

#include <app/Application.h>
#include <app/Roster.h>
#include <interface/Alert.h>
#include <interface/Bitmap.h>
#include <storage/Resources.h>
#include <iostream>

#include "TopView.h"
#include "ShellWindow.h"
#include "../olib/Colors.h"


TopView::TopView(BRect rect, char *name)
    : BView(rect, name, B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW)
{
	fMouseDown = false;	// avoids cursor confusion

	/* --- Load background image --- */
	BResources	*Resourcen = be_app->AppResources();
	size_t		thesize;
	BMessage	msg;	
	char		*buf = (char *)Resourcen->LoadResource('BBMP', 1, &thesize);
	if(buf != NULL) {
		msg.Unflatten(buf);
		backBit = new BBitmap( &msg );
		msg.MakeEmpty();
	} else {
		cout << "*** Resource not found" << endl;
	}
}


TopView::~TopView( void )
{
	delete backBit;
}

/* ======================================================================= */

void TopView::AttachedToWindow( void ) 
{
	#ifdef	DEBUG
		cout << "View AttachedToWindow" << endl;
	#endif
	
	// Set Colors
    SetViewColor(greyCol);
    SetLowColor(lightGreyCol);

	BView::AttachedToWindow();
	
	// Paint Background bitmap
	SetViewBitmap(backBit);
}


void TopView::AllAttached()
{
	#ifdef	DEBUG
		cout << "View AllAttached" << endl;
	#endif

	BView::AllAttached();
}


void TopView::Draw(BRect updateRect)
{
	#ifdef	DEBUG
		cout << "View Draw" << endl;
	#endif

	// This line drawing is only for adding a
	// subtle 3D look to the bottom view border.
	BPoint	start((updateRect.left), (updateRect.bottom));
	BPoint	end((updateRect.right), (updateRect.bottom));

	SetPenSize( 1.0 );
	StrokeLine( start, end, B_SOLID_LOW );	// LowColor is defined above
	
	BView::Draw(updateRect);
}


void TopView::MessageReceived(BMessage* message)
{
	#ifdef	DEBUG
		message->PrintToStream();
	#endif
	
	switch(message->what)
	{			
		default:
			BView::MessageReceived(message);
	}
}


void TopView::MouseDown(BPoint where)
{
	fMouseDown = true;
	fMouseOffset = where;

	SetMouseEventMask(B_POINTER_EVENTS, B_NO_POINTER_HISTORY|B_SUSPEND_VIEW_FOCUS|B_LOCK_WINDOW_FOCUS );

	BView::MouseDown(where);

	if(Window()->IsActive() == false) {
		Window()->Activate();
		fMouseDown = false;
	}
}


void TopView::MouseUp(BPoint where)
{
	fMouseDown = false;
	BView::MouseUp(where);
}


void TopView::MouseMoved( BPoint where, uint32 code, const BMessage *a_message)
{
	if(fMouseDown) {
		BPoint win_pos = Window()->Frame().LeftTop();
		Window()->MoveTo( win_pos.x + where.x - fMouseOffset.x, win_pos.y + where.y - fMouseOffset.y );
	} else {
		if(code == B_ENTERED_VIEW) {	/* --- Display meaning of modifier keys --- */
			bool	filenameonly = (modifiers() & B_SHIFT_KEY);
			bool	appendrefs = (modifiers() & B_LEFT_CONTROL_KEY);
			bool	prependrefs = (modifiers() & B_COMMAND_KEY);
			
			// Parse pairs of keypresses first, or they won't be recognized
			if( (filenameonly == true) && (appendrefs == true) ) {
				((ShellWindow *)Window())->DisplayStatus("Drop + SHIFT + LEFT-CTRL appends filenames without path at the end.", yelCol);
			} else if( (filenameonly == true) && (prependrefs == true) ) {
				((ShellWindow *)Window())->DisplayStatus("Drop + SHIFT + COMMAND_KEY prepends filenames without path at start.", yelCol);
			} else if(filenameonly) {
				((ShellWindow *)Window())->DisplayStatus("Drop + SHIFT means strip off path from filename.", yelCol);
			} else if(appendrefs) {
				((ShellWindow *)Window())->DisplayStatus("Drop + LEFT_CTRL appends filenames at the end of command.", yelCol);
			} else if(prependrefs) {
				((ShellWindow *)Window())->DisplayStatus("Drop + COMMAND_KEY prepends filenames at start of command.", yelCol);
			}
		}
		BView::MouseMoved(where, code, a_message);
	}
}
