/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++                ConfigView.cpp                  +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++         Copyright © 2007 Ove Peter.            +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

#include <interface/Window.h>
#include <support/String.h>

#include "ConfigView.h"
#include "SaverApp.h"

#include "../olib/OStringView.h"
#include "../olib/Colors.h"

ConfigView::ConfigView(BRect rect, char *name, BString *commtext, bool *black_screen, bool *repeat_command)
    : BView(rect, name, B_FOLLOW_ALL, B_WILL_DRAW)
{	
	command = commtext;
	black = black_screen;
	repeat = repeat_command;
	
	fMouseDown = false;	// avoids cursor confusion

	/* Construct controls */
	float width = rect.Width();
	float height = rect.Height();

	AddChild(new OStringView(BRect(5,10,width,30), "", APP_NAME" "APP_VERSION));
	AddChild(new BStringView(BRect(5,32,width,52), "", "Copyright © 2007 Ove Peter"));

	BStringView *warn = new BStringView(BRect(5,58,width,70), "warn", "WARNING: Alpha software, use with caution!");
	warn->SetHighColor(redCol);
	AddChild(warn);

	BStringView *labelView = new BStringView(BRect(5,70,width,105), "", "Input new shell command and press ENTER:");
	labelView->SetHighColor(blueCol);
	AddChild(labelView);
	
	commBar = new OTextControl(BRect(5,110,width,140), "commbar", NULL, command->String(), new BMessage(COMMAND_CHANGED));
	AddChild(commBar);
	
	
	OCheckBox *nokillCB = new OCheckBox(BRect(5,155,width,170), "nokill_cb", "Wait for program-exit when stopping Saver", NULL);
	nokillCB->SetValue(B_CONTROL_ON);
	nokillCB->SetEnabled(false);
	AddChild(nokillCB);

	repeatCB = new OCheckBox(BRect(5,175,width,190), "repeat_cb", "Repeatedly execute while ScreenSaver runs", new BMessage(REPEAT));
	repeatCB->SetValue(*repeat);
	// repeatCB->SetEnabled(false);
	AddChild(repeatCB);

	blackCB = new OCheckBox(BRect(5,195,width,210), "black_cb", "Black screen background (else white)", new BMessage(BLACK_SCREEN));
	blackCB->SetValue(*black);
	AddChild(blackCB);

		
	statView = new OStringView(BRect(5,(height - 20),width,height), "", "http://www.planetmir.de/");
	AddChild(statView);
}


ConfigView::~ConfigView( void )
{

}

/* ======================================================================= */

void ConfigView::AttachedToWindow( void ) 
{
    SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));

	commBar->SetTarget(this);
	blackCB->SetTarget(this);
	repeatCB->SetTarget(this);

	BView::AttachedToWindow();
}


void ConfigView::AllAttached()
{
	BView::AllAttached();
}


void ConfigView::Draw(BRect updateRect)
{	
	BView::Draw(updateRect);
}


void ConfigView::MessageReceived(BMessage* message)
{
	switch(message->what)
	{
		case COMMAND_CHANGED:
			command->SetTo(commBar->Text());
			break;

		case BLACK_SCREEN:
			*black = (!*black) ? true : false;
			break;

		case REPEAT:
			*repeat = (!*repeat) ? true : false;
			break;
						
		default:
			BView::MessageReceived(message);
			break;
	}
}


void ConfigView::MouseDown(BPoint where)
{
	MakeFocus();
	
	fMouseDown = true;
	fMouseOffset = where;

	SetMouseEventMask(B_POINTER_EVENTS, B_NO_POINTER_HISTORY|B_SUSPEND_VIEW_FOCUS|B_LOCK_WINDOW_FOCUS );

	BView::MouseDown(where);

	if(Window()->IsActive() == false) {
		Window()->Activate();
		fMouseDown = false;
	}
}


void ConfigView::MouseUp(BPoint where)
{
	fMouseDown = false;
	BView::MouseUp(where);
}


void ConfigView::MouseMoved( BPoint where, uint32 code, const BMessage *a_message)
{
	if(fMouseDown) {
		BPoint win_pos = Window()->Frame().LeftTop();
		Window()->MoveTo( win_pos.x + where.x - fMouseOffset.x, win_pos.y + where.y - fMouseOffset.y );
	} else {
		BView::MouseMoved(where, code, a_message);
	}
}
