/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++                ShellSaver.cpp                  +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++         Copyright © 2007 Ove Peter.            +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

#include <storage/Resources.h>
#include <interface/Bitmap.h>
#include <interface/StringView.h>

#include <stdio.h>			// for popen()

#include "ShellSaver.h"
#include "SaverApp.h"
#include "ConfigView.h"

extern "C" { 
	_EXPORT 
	BScreenSaver *instantiate_screen_saver( BMessage *msg, image_id id ); 
}


BScreenSaver *instantiate_screen_saver(BMessage *msg, image_id image) 
{ 
	return new ShellSaver(msg, image);
} 


ShellSaver::ShellSaver(BMessage *archive, image_id id) :
	BScreenSaver(archive, id)
{
	/* Get the settings from *archive */
	if(archive->FindString("command", &command) != B_OK) {
		command = "fortune";
	}
	if(archive->FindBool("black_screen", &black_screen) != B_OK) {
		black_screen = true;
	}
	if(archive->FindBool("repeat", &repeat) != B_OK) {
		repeat = false;
	}
}


ShellSaver::~ShellSaver()
{

}


void ShellSaver::StartConfig(BView *view) 
{
	configView = new ConfigView(view->Bounds(), "configview", &command, &black_screen, &repeat);
	view->AddChild(configView);
} 


void ShellSaver::StopConfig()
{

}


status_t ShellSaver::SaveState(BMessage *state) const
{
	if(state->ReplaceString("command", command) == B_NAME_NOT_FOUND) {
		state->AddString("command", command);
	}
	if(state->ReplaceBool("black_screen", black_screen) == B_NAME_NOT_FOUND) {
		state->AddBool("black_screen", black_screen);
	}
	if(state->ReplaceBool("repeat", repeat) == B_NAME_NOT_FOUND) {
		state->AddBool("repeat", repeat);
	}

	return B_OK;
}


status_t ShellSaver::StartSaver(BView *view, bool preview)
{
	// Store if we are in preview mode (important!).
	is_preview = preview;

	// Construct Text View and set colors.	
	tView = new BTextView(view->Frame(),"shellview",view->Bounds(),B_FOLLOW_ALL, B_WILL_DRAW);

	if(black_screen) {
		tView->SetViewColor(0, 0, 0);
		rgb_color	fontCol = {255, 255, 255, 255};
		tView->SetFontAndColor(be_fixed_font, B_FONT_ALL, &fontCol);
	} else {
		tView->SetViewColor(255, 255, 255);
		rgb_color	fontCol = {0, 0, 0, 255};
		tView->SetFontAndColor(be_fixed_font, B_FONT_ALL, &fontCol);
	}

	view->AddChild(tView);

	return B_OK;
}


void ShellSaver::StopSaver()
{
	BScreenSaver::StopSaver();
}


void ShellSaver::Draw(BView *view, int32 frame)
{
	if(frame == 0) {
		// Display info for user.
		tView->Insert("\n"APP_NAME", executing command \"");
		tView->Insert(command.String());
		tView->Insert("\"...\n\n\n");
		/* --- Execute command.
		++ --- A bit slow in Draw(), but one sees the progress
		++ --- as opposed to starting in StartSaver().
		*/
		if(!is_preview) {
			exeThreadID = -1;
			exeThreadID = spawn_thread((thread_func )ExeThread, "LaunchShellCommand", B_NORMAL_PRIORITY, NULL);
			if(exeThreadID >= 0) {
				resume_thread(exeThreadID);
			}
			// todo
			ExecuteCommand();
		}
	}
	
	// If repeat is on, repeat command ;)
	if(!is_preview && repeat) {
		tView->Insert("\n\n\n");
		ExecuteCommand();
	}
	
	// In preview mode, scroll current command.
	if(is_preview) {
		tView->Insert(command.String());
		tView->Insert("\n");
		tView->ScrollToOffset(tView->TextLength());
		snooze(10000);
	}
}

/* ================================================================================= */

int ShellSaver::ExecuteCommand(void)
{
	int 		shellExit = -1;
	FILE		*fd;
	char		buff[256];
	bool		amperExist = false;
	
	/* --- Prepare the command string --- */
	
	// Search for an amper ( &) in last 2 chars and remove
	if( command.FindFirst(" &", (command.CountChars() - 2)) != B_ERROR ) {
		amperExist = true;
		command.RemoveLast(" &");
	}
	// Merge stderr with stdout!
	command << " 2>&1";
	// If an amper was removed before, re-apply
	if(amperExist) {
		command << " &";
	}
	
	
	/* --- Open pipe to shell command an display stdout --- */
	
	fd = popen(command.String(), "r");
	
	if(fd)
	{
		while(fgets(buff, 256, fd) != NULL)
		{
			tView->Insert((const char *)buff);
			tView->ScrollToOffset(tView->TextLength());
		}
	}
	
	shellExit = pclose(fd);		// Close filepointer.
	
	return shellExit;	
}


int32 ShellSaver::ExeThread()
{	// todo (mandatory)
	return 0;
	// return ExecuteCommand();
}
