/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++                  InputWin.cpp                  +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++         Copyright © 2007 Ove Peter.            +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

#define BUILDING_OLIB

#include <interface/Screen.h>
#include <support/SupportDefs.h>

#include "InputWin.h"

#define INPUT_WIDTH		300
#define INPUT_HEIGHT	35


InputWindow::InputWindow(BHandler *target, const char *title, window_type type, uint32 flags, uint32 workspaces) :
		BWindow(BRect(100,100,100,100), title, type, flags, workspaces)
{
	caller = target;

	/* --- Place this window in the middle of the screen --- */
	BScreen *theScreen = new BScreen();
	BRect theSize = theScreen->Frame();
	float theWidth = theSize.right;
	float theHeight = theSize.bottom;

	theSize.left = (theWidth - INPUT_WIDTH) / 2;
	theSize.top = (theHeight - INPUT_HEIGHT) / 2;
	theSize.right = theSize.left + INPUT_WIDTH;
	theSize.bottom = theSize.top + INPUT_HEIGHT;			
		
	ResizeTo(theSize.Width(), theSize.Height());
	MoveTo(theSize.left, theSize.top);
	
	
	/* --- Construct Controls --- */
	BFont	font(be_plain_font);
	if(font.Size() > 12.0) font.SetSize(12.0);
	
	// Top View
	theSize = Bounds();
	BView	*topView = new BView(theSize, "InputView", B_FOLLOW_ALL_SIDES, B_NAVIGABLE | B_WILL_DRAW);
	topView->SetViewColor(240, 205, 0, 255);
	AddChild(topView);
		
	// Text input Bar
	// TODO: use an OTextControl when it has a label parameter !
	BRect	textRect(5, theSize.top+5, theSize.right-40, theSize.Height()-5); 
	textBar = new BTextControl(textRect, "inputbar", title, NULL, NULL);
	textBar->SetFont(&font, B_FONT_ALL);			// Set label text size
	//textBar->TextView()->SetFontAndColor(&font);	// Set TextControl text size
	textBar->SetDivider(75);
	topView->AddChild(textBar);
	textBar->MakeFocus(true);
	
	// Set - Button
	BRect	butRect(theSize.right-30, theSize.top+5, theSize.right-5, theSize.Height()-10);
	setBut = new BButton(butRect, "setbut", "set", new BMessage(SET_BUT));
	setBut->SetFont(&font, B_FONT_ALL);
	setBut->MakeDefault(true);
	topView->AddChild(setBut);
	
	Show();
}


InputWindow::~InputWindow(void)
{

}

/* ======================================================================================================================================== */

bool InputWindow::QuitRequested(void)
{
	return(true);
}


void InputWindow::MessageReceived(BMessage *message)
{
	// :::::::
	// message->PrintToStream();
	
	switch(message->what)
	{
	case SET_BUT:
	{
		/* Get the value from textcontrol */
		const char *value = textBar->Text();
		
		/* Prepare a message to the BHandler who spawned us */
		BMessage	msg(INWIN_REPLY);
		msg.AddString("value", value);
		
		/* Get looper of target BHandler and post the message to it */
		status_t	err;
		err = ((BLooper *)caller->Looper())->PostMessage(&msg, caller);
		if(err != B_OK) {
			textBar->SetText("ERROR sending message to handler!");
			snooze(2000000);
		}
		
		/* Quit this window */
		Lock();
		Quit();
	}	break;
		
	default:
		BWindow::MessageReceived(message);
		break;
	}
}

/* ========================================================================= */
