/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++                OMenuBar.cpp                    +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++         Copyright © 2007 Ove Peter.            +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

#define BUILDING_OLIB

#include <interface/Window.h>		// for Window()

#include "OMenuBar.h"

OMenuBar::OMenuBar(BRect frame, const char *name, uint32 resizingMode, menu_layout layout, bool resizeToFit)
	: BMenuBar(frame, name, resizingMode, layout, resizeToFit)
{
	fMouseDown = false;
	
	/* --- Style the MenuBar, Font, etc. --- */
	SetBorder(B_BORDER_FRAME);		// B_BORDER_FRAME	B_BORDER_CONTENTS	B_BORDER_EACH_ITEM
	BFont	font(be_plain_font);
	if(font.Size() > 12.0)			
		font.SetSize(12.0);
	SetFont(&font, B_FONT_ALL);
}


OMenuBar::~OMenuBar(void)
{

}

/* ====================================================================== */

void OMenuBar::FrameResized(float width, float height)
{
	BMenuBar::FrameResized(width, height);
	
	Draw(Bounds());	// Invalidate() seems to be higher on CPU ??!!
}


void OMenuBar::MouseDown(BPoint point)
{
	fMouseDown = true;
	fMouseOffset = point;

	SetMouseEventMask(B_POINTER_EVENTS, B_NO_POINTER_HISTORY|B_SUSPEND_VIEW_FOCUS|B_LOCK_WINDOW_FOCUS );

	BMenuBar::MouseDown( point );	// call the MenuBar version of MouseDown()
}


void OMenuBar::MouseUp(BPoint where)
{
	fMouseDown = false;
	BMenuBar::MouseUp( where );
}


void OMenuBar::MouseMoved( BPoint where, uint32 code, const BMessage *a_message)
{
	if(fMouseDown) {
		BPoint win_pos = Window()->Frame().LeftTop();
		Window()->MoveTo( win_pos.x + where.x - fMouseOffset.x, win_pos.y + where.y - fMouseOffset.y );
	} else {
		BMenuBar::MouseMoved( where, code, a_message);
	}
}

/* ====================================================================== */

void OMenuBar::SetMouseOff(void)
{	// Prevent window-move
	fMouseDown = false;
}
