/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++                  OTextControl.h                +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++         Copyright © 2007 Ove Peter.            +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifndef O_TEXT_CONTROL_H
#define O_TEXT_CONTROL_H

#ifdef BUILDING_OLIB
	#define OLIB_IMPEXP	__declspec(dllexport)
#else
	#define OLIB_IMPEXP	__declspec(dllimport)
#endif

#include <interface/Control.h>
#include <kernel/OS.h>

#include "OTextView.h"

class OLIB_IMPEXP OTextControl : public BControl {
	public:
		/* --- Constructor & Deconstructor --- */
				OTextControl(BRect frame, const char *name,
						const char *label, const char *text,
						BMessage *message = NULL,
						uint32 resizingMode = B_FOLLOW_LEFT_RIGHT,
						uint32 flags = B_WILL_DRAW | B_NAVIGABLE | B_FRAME_EVENTS,
						bool do_autocomplete = true,
						bool do_animation = true);
				~OTextControl( void );					// B_FRAME_EVENTS is absolutely needed for resizing!!!!
		/* --- Hook functions and more --- */
		virtual void	AllAttached( void );			// Set Background colors here.
		virtual void	WindowActivated(bool active);	// En/disables the TextView on Window focus.
		virtual void	MakeFocus(bool flag = true);	/* If flag true, TextControl gets focus and all text is selected.
														-- If flag false, send model msg and loose focus. */
		virtual void	KeyDown(const char *bytes, int32 numBytes); // Everything that arrives here only goes to our OTextView child.
		virtual void	FrameResized(float width, float height);	// On frame resize, store dimensions.
		virtual void	Draw(BRect updateRect);			// Draws the Dano-alike TextControl border.
		/* --- Individual functions for B(Text)Control behaviour, etc. --- */
		OTextView			*TextView( void );			// Returns the TextView Child of this control.
		virtual void		SetEnabled(bool enabled);
		virtual status_t	SetMessage(BMessage *message);		// Changes the model message.
		virtual void		SetModificationMessage(BMessage *message);	// dito, for BTextControl compatibility.
		BMessage			*Message(void) const;				// Returns the model message.
		BMessage			*ModificationMessage(void) const;	// dito.
		virtual status_t	Invoke(BMessage *message = NULL);	// Sends the model message.
		/* --- Individual functions for text manipulation --- */
		virtual void	AppendText(BString text);
		virtual void	AppendText(const char *text);
		virtual void	PrependText(BString text);
		virtual void	PrependText(const char *text);
		virtual void	SetText(BString text);
		virtual void	SetText(const char *text);
		const char		*Text( void );
		virtual void	SelectAll( void );
		virtual void	SetAlignment(alignment forLabel, alignment forText);	// Alignment for text like in BTextControl
		/* --- Individual functions for auto-complete --- */
		virtual void	SetAutoComplete(bool doIt = true);
		bool			IsAutoComplete( void );
		virtual void	AddAutoText(BString *text, int32 index);
		virtual void	AddAutoText(BString *text);
		virtual void	RemoveAutoItem(int32 index);

	private:
		/* --- Private functions --- */
		void			focus_animation();
		static int32	_focus(void *arg);
		/* --- Private datamembers --- */
		thread_id	focus_thread;
		bool 		keep_going;
	
		OTextView	*tView;				// TextView Child.		
		float		vwidth, vheight;	// Width and Height of whole control.
		BRect		vRect;				// Rectangle bounds of the control view.
		bool		is_enabled;			// Wether this view is enabled.
		BMessage	*modelMsg;			// The model message that's sent on actions.
};

#endif	// O_TEXT_CONTROL_H
