/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++                   OTextView.h                  +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* +++         Copyright © 2007 Ove Peter.            +++ */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifndef O_TEXT_VIEW_H
#define O_TEXT_VIEW_H

#ifdef BUILDING_OLIB
	#define OLIB_IMPEXP	__declspec(dllexport)
#else
	#define OLIB_IMPEXP	__declspec(dllimport)
#endif

#include <interface/TextView.h>
#include <support/List.h>

class OLIB_IMPEXP OTextView : public BTextView {
	public:
		/* --- Constructor & Deconstructor --- */
				OTextView(BRect frame, const char *name,
						BRect textRect,
						uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP,
						uint32 flags = B_WILL_DRAW | B_NAVIGABLE | B_FRAME_EVENTS | B_PULSE_NEEDED,
						BControl *container = NULL,
						bool do_autocomplete = false);
				~OTextView( void );
		/* --- Hook functions --- */
		virtual void	KeyDown(const char *bytes, int32 numBytes);
		virtual void	MakeFocus(bool focused = true);	// Will also trigger the parent BControl to send model msg
		/* --- Individual functions for text manipulation, etc --- */
		virtual void	AppendText(BString text);
		virtual void	AppendText(const char *text);
		virtual void	PrependText(BString text);
		virtual void	PrependText(const char *text);
		/* --- Individual functions for auto-complete --- */
		virtual void	SetAutoComplete(bool doIt = true);
		bool			IsAutoComplete( void );
		virtual void	AddAutoText(BString *text, int32 index);
		virtual void	AddAutoText(BString *text);
		virtual void	RemoveAutoItem(int32 index);

	protected:
		virtual void	InsertText(const char *text, int32 length, int32 offset, const text_run_array *runs);

	private:
		/* --- Private datamembers --- */
		BControl	*parent;		// The parent OTextControl container object.
		bool		doAuto;			// Wether auto-complete should be done.
		BList		*autoList;		// A list which holds the auto-complete strings.
		int32		triggerNum;		// After how many inputted letters autocomplete should be done.
		bool		hl;				// Wether text has to be highlighted.
	 	int32		hl_offset, hl_end;	// Start- and End-Offset of Text to Be highlighted.

	 	int32		currAutoItem;		// Number of last Auto item of up-down action.
};

#endif	// O_TEXT_VIEW_H
